'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var babel = require('@babel/core');

/* eslint-disable no-param-reassign */

function isSimpleStyleSheetObjectExpression(path) {
  if (!path.isImmutable()) {
    if (path.isPure()) {
      // it can contain string literals so try to evaluate it
      const expressionResult = path.evaluate();

      if (expressionResult.confident) {
        // we know the result, check its immutablity
        const {
          value
        } = expressionResult; // value can be only object in that case freeze it

        if (typeof value === 'object') {
          return true;
        }

        return false;
      }
    }

    return false;
  }

  return true;
}

function isStylesObjectProperty(path) {
  const {
    node
  } = path;

  if (babel.types.isIdentifier(node.key)) {
    return node.key.name === 'styles';
  }

  if (babel.types.isTemplateLiteral(node.key)) {
    // try to evaluate
    const keyExpr = path.get('key');
    const result = keyExpr.evaluate();
    return result.confident && result.value === 'styles';
  }

  return false;
}

function hostVisageStylesPlugin({
  types,
  template
}) {
  const freezeStyleSheet = template('Object.freeze(STYLE_SHEET)');
  return {
    name: 'visage',
    // eslint-disable-next-line global-require
    // inherits: require('@babel/plugin-syntax-jsx'),
    visitor: {
      ObjectProperty(path) {
        // is a part of jsx spread attribute?
        if (!path.findParent(p => types.isJSXSpreadAttribute(p))) {
          return;
        }

        if (!isStylesObjectProperty(path)) {
          return;
        }

        const valuePath = path.get('value');

        if (isSimpleStyleSheetObjectExpression(valuePath)) {
          // replace with freeze
          valuePath.replaceWith(freezeStyleSheet({
            STYLE_SHEET: valuePath.node
          })); // @ts-ignore hoist doesn't need a scope argument

          valuePath.hoist();
        }
      },

      // process only jsx styles attribute
      JSXAttribute(path) {
        const {
          node
        } = path;

        if (node.name.name !== 'styles') {
          return;
        }

        if (types.isJSXExpressionContainer(node.value) && types.isObjectExpression(node.value.expression)) {
          const expressionPath = path.get('value.expression');

          if (isSimpleStyleSheetObjectExpression(expressionPath)) {
            // replace with freeze
            expressionPath.replaceWith(freezeStyleSheet({
              STYLE_SHEET: expressionPath.node
            })); // @ts-ignore - hoist doesn't need a scope argument

            expressionPath.hoist();
          }
        }
      }

    }
  };
}

exports.default = hostVisageStylesPlugin;
//# sourceMappingURL=babel-plugin-visage.cjs.development.js.map
